package net.minecraft.src;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import net.minecraft.client.Minecraft;

// SDK
@SuppressWarnings({ "rawtypes", "unchecked" })
public class mod_SdkFlasher extends SdkBaseModMp
{
	private static Map<SdkLitBlock, Integer> litBlocks = new HashMap<SdkLitBlock, Integer>();
	
	@Override
	public String Version()
	{
		return "1.3_01v5";
	}
	
	@Override
	public void OnTickInGameTick(Minecraft minecraft)
	{
		if(minecraft.theWorld != null)
		{
			Iterator<Map.Entry<SdkLitBlock, Integer>> it = litBlocks.entrySet().iterator();
			while (it.hasNext()) {
			    Map.Entry<SdkLitBlock, Integer> entry = it.next();
			    
			    int duration = entry.getValue() - 1;
			    
			    if (duration <= 0)
			    {
			    	unlightBlock(minecraft.theWorld, entry.getKey());
			        it.remove();
			    }
			    else
			    	entry.setValue(duration);
			}
		}
	}
	
	public static void LightEntity(World world, Entity entity, int lightLevel, int duration)
	{
		LightBlock(world, (int)entity.posX, (int)entity.posY, (int)entity.posZ, lightLevel, duration);
	}
	
	public static void LightBlock(World world, int blockX, int blockY, int blockZ, int lightLevel, int duration)
	{
		LightBlock(world, new SdkPoint3d<Integer>(blockX, blockY, blockZ), lightLevel, duration);
	}
	
	private static void LightBlock(World world, SdkPoint3d<Integer> blockLocation, int lightLevel, int duration)
	{
		clearLitBlocks(world);
		
		int radius = lightLevel - 1;
		int diameter = radius * 2 + 1;
		int[] storedLightValues = new int[diameter * diameter * diameter];
		
		for(int x = -radius; x <= radius; x++)
			for(int y = -radius; y <= radius; y++)
				for(int z = -radius; z <= radius; z++)
				{
					int index = ((z + radius) * diameter * diameter) + ((y + radius) * diameter) + (x + radius);
					storedLightValues[index] = -1;
					
					if(world.getBlockId(blockLocation.x + x, blockLocation.y + y, blockLocation.z + z) == 0)
					{
						int distance = Math.abs(x) + Math.abs(y) + Math.abs(z);
						int newLightLevel = Math.max(0, lightLevel - distance);
						
						int oldLightLevel = world.getBlockLightValue(blockLocation.x + x, blockLocation.y + y, blockLocation.z + z);
						
						if(newLightLevel > oldLightLevel)
						{
							storedLightValues[index] = oldLightLevel;
							world.setLightValue(EnumSkyBlock.Block, blockLocation.x + x, blockLocation.y + y, blockLocation.z + z, newLightLevel);
						}
					}
				}
		
		SdkLitBlock litBlock = new SdkLitBlock(blockLocation, storedLightValues, lightLevel);
		litBlocks.put(litBlock, duration);
	}
	
	private static void clearLitBlocks(World world)
	{
		Iterator<Map.Entry<SdkLitBlock, Integer>> it = litBlocks.entrySet().iterator();
		while (it.hasNext()) {
		    Map.Entry<SdkLitBlock, Integer> entry = it.next();
		    
	    	unlightBlock(world, entry.getKey());
	        it.remove();
		}
	}

	private static void unlightBlock(World world, SdkLitBlock litBlock)
	{
		SdkPoint3d<Integer> blockLocation = litBlock.getBlockLocation();
		int[] lightValues = litBlock.getLightValues();
		int lightLevel = litBlock.getLightLevel();
		int radius = lightLevel - 1;
		int diameter = radius * 2 + 1;
		
		for(int x = -radius; x <= radius; x++)
			for(int y = -radius; y <= radius; y++)
				for(int z = -radius; z <= radius; z++)
				{
					int index = ((z + radius) * diameter * diameter) + ((y + radius) * diameter) + (x + radius);
					
					if(lightValues[index] != -1 && world.getBlockId(blockLocation.x + x, blockLocation.y + y, blockLocation.z + z) == 0)
						world.setLightValue(EnumSkyBlock.Block, blockLocation.x + x, blockLocation.y + y, blockLocation.z + z, lightValues[index]);
				}
	}

	@Override
	public void SetRecipes(CraftingManager craftingmanager) { }

	@Override
	public void OnMainMenu() { }

	@Override
	public void InitBlocks() { }
}
